@@ -21,17 +21,21 @@
 #pylint: disable=invalid-name
 #pylint: disable=too-many-nested-blocks
 
-from typing import List, Tuple, Optional
+from typing import List, Tuple, Optional, TYPE_CHECKING
 import copy
 
 import numpy
+import fqe.settings
+from fqe.lib.wick import _wickfill
 
 Mapping = Tuple[List[Tuple[str, str]], List[Tuple[str, bool, int]], float]
 
+if TYPE_CHECKING:
+    from numpy import ndarray as Nparray
 
-def wick(target: str,
-         data: List[numpy.ndarray],
-         spinfree: Optional[bool] = True) -> numpy.ndarray:
+
+def wick(target: str, data: List['Nparray'],
+         spinfree: Optional[bool] = True) -> 'Nparray':
     """
     Original and target are written in a similar notation to OpenFermion
     operators.
@@ -49,19 +53,25 @@ def wick(target: str,
     correspond to n-body strings
 
     Args:
-        target (string) - specifies the operator list
+        target (string): specifies the operator list
 
-        data (List[numpy.ndarray]) - a list of particle RDMs
+        data (List['Nparray']): a list of particle RDMs
 
-        spinfree (bool) - whether the RDMs are spinfree
+        spinfree (bool): whether the RDMs are spinfree
 
     Returns:
-        (numpy.ndarray) - RDM after performing Wick's theorem
+        ('Nparray'): RDM after performing Wick's theorem
     """
 
     def process_string(inp: str) -> List[Tuple[str, bool, int]]:
-        """ input is the string. Returns a list of indices described by index
-        label, dagger (or not), and spin numbers
+        """Return a list of indices described by index label, dagger (or not),
+        and spin numbers
+
+        Args:
+            inp (str): input string
+
+        Retunrs:
+            list: List of indices
         """
         out: List[Tuple[str, bool, int]] = []
         used: List[str] = []
@@ -181,15 +191,15 @@ def wick(target: str,
             delta.append((indices[j_t[0]], indices[j_t[1]]))
 
         if irank > 0:
-            wickfill(out, data[irank - 1], sources, term[2], delta)
+            out = wickfill(out, data[irank - 1], sources, term[2], delta)
         else:
-            wickfill(out, None, sources, term[2], delta)
+            out = wickfill(out, None, sources, term[2], delta)
 
     return out
 
 
-def wickfill(target: numpy.ndarray, source: numpy.ndarray, indices: List[int],
-             factor: float, delta: List[Tuple[int, int]]) -> None:
+def wickfill(target: 'Nparray', source: Optional['Nparray'], indices: List[int],
+             factor: float, delta: List[Tuple[int, int]]) -> 'Nparray':
     """
     This function is an internal utility that fills in custom RDMs using
     particle RDMs. The result of Wick's theorem is passed as lists (indices
@@ -197,25 +207,42 @@ def wickfill(target: numpy.ndarray, source: numpy.ndarray, indices: List[int],
     target.
 
     Args:
-        target (numpy.ndarray) - output array that stores reordered RDMs
+        target ('Nparray'): output array that stores reordered RDMs
 
-        source (numpy.ndarray) - input array that stores one of the particle RDMs
+        source ('Nparray'): input array that stores one of the particle RDMs
 
-        indices (List[int]) - index mapping
+        indices (List[int]): index mapping
 
-        factor (float) - factor associated with this contribution
+        factor (float): factor associated with this contribution
 
-        delta (List[Tuple[int, int]]) - Kronecker delta's due to Wick's theorem
+        delta (List[Tuple[int, int]]): Kronecker delta's due to Wick's theorem
+
+    Returns:
+        target ('Nparray'): Returns the output array. If target in the \
+            input Args was not C-contigious, this can be a new numpy object.
     """
+    delta_flat = None
+    if len(delta) > 0:
+        delta_flat = numpy.asarray(delta, dtype=numpy.uint32)
+
+    indices_flat = None
+    if indices is not None:
+        indices_flat = numpy.array(indices, dtype=numpy.uint32)
+
     norb = target.shape[0]
     srank = len(source.shape) // 2 if source is not None else 0
     trank = len(target.shape) // 2
+
+    if fqe.settings.use_accelerated_code:
+        return _wickfill(target, source, indices_flat, factor, delta_flat)
+
+    #else:
     assert srank * 2 == len(indices)
     if srank == 0 and trank == 1:
         assert len(delta) == 1
         for i in range(norb):
             target[i, i] += factor
-    elif srank == 1 and trank == 1:
+    elif srank == 1 and trank == 1 and source is not None:
         assert not delta
         mat = {}
         for i in range(norb):
@@ -238,7 +265,7 @@ def wickfill(target: numpy.ndarray, source: numpy.ndarray, indices: List[int],
                         if mat[delta[0][0]] == mat[delta[0][1]] and \
                             mat[delta[1][0]] == mat[delta[1][1]]:
                             target[i, j, k, l] += factor
-    elif srank == 1 and trank == 2:
+    elif srank == 1 and trank == 2 and source is not None:
         assert len(delta) == 1
         mat = {}
         for i in range(norb):
@@ -252,7 +279,7 @@ def wickfill(target: numpy.ndarray, source: numpy.ndarray, indices: List[int],
                         if mat[delta[0][0]] == mat[delta[0][1]]:
                             target[i, j, k, l] += \
                                 factor * source[mat[indices[0]], mat[indices[1]]]
-    elif srank == 2 and trank == 2:
+    elif srank == 2 and trank == 2 and source is not None:
         assert not delta
         mat = {}
         for i in range(norb):
@@ -287,7 +314,7 @@ def wickfill(target: numpy.ndarray, source: numpy.ndarray, indices: List[int],
                                     mat[delta[1][0]] == mat[delta[1][1]] and \
                                     mat[delta[2][0]] == mat[delta[2][1]]:
                                     target[i, j, k, l, o, p] += factor
-    elif srank == 1 and trank == 3:
+    elif srank == 1 and trank == 3 and source is not None:
         assert len(delta) == 2
         mat = {}
         for i in range(norb):
@@ -307,7 +334,7 @@ def wickfill(target: numpy.ndarray, source: numpy.ndarray, indices: List[int],
                                     target[i, j, k, l, o, p] += \
                                         factor * source[mat[indices[0]],
                                                         mat[indices[1]]]
-    elif srank == 2 and trank == 3:
+    elif srank == 2 and trank == 3 and source is not None:
         assert len(delta) == 1
         mat = {}
         for i in range(norb):
@@ -328,7 +355,7 @@ def wickfill(target: numpy.ndarray, source: numpy.ndarray, indices: List[int],
                                                         mat[indices[1]],
                                                         mat[indices[2]],
                                                         mat[indices[3]]]
-    elif srank == 3 and trank == 3:
+    elif srank == 3 and trank == 3 and source is not None:
         assert not delta
         mat = {}
         for i in range(norb):
@@ -375,7 +402,7 @@ def wickfill(target: numpy.ndarray, source: numpy.ndarray, indices: List[int],
                                         and mat[delta[3][0]] == mat[delta[3][1]]:
                                             target[i, j, k, l, o, p, q, r] += \
                                                 factor
-    elif srank == 1 and trank == 4:
+    elif srank == 1 and trank == 4 and source is not None:
         assert len(delta) == 3
         mat = {}
         for i in range(norb):
@@ -400,7 +427,7 @@ def wickfill(target: numpy.ndarray, source: numpy.ndarray, indices: List[int],
                                             target[i, j, k, l, o, p, q, r] += \
                                                 factor * source[mat[indices[0]],
                                                                 mat[indices[1]]]
-    elif srank == 2 and trank == 4:
+    elif srank == 2 and trank == 4 and source is not None:
         assert len(delta) == 2
         mat = {}
         for i in range(norb):
@@ -426,7 +453,7 @@ def wickfill(target: numpy.ndarray, source: numpy.ndarray, indices: List[int],
                                                                 mat[indices[1]],
                                                                 mat[indices[2]],
                                                                 mat[indices[3]]]
-    elif srank == 3 and trank == 4:
+    elif srank == 3 and trank == 4 and source is not None:
         assert len(delta) == 1
         mat = {}
         for i in range(norb):
@@ -453,7 +480,7 @@ def wickfill(target: numpy.ndarray, source: numpy.ndarray, indices: List[int],
                                                                 mat[indices[3]],
                                                                 mat[indices[4]],
                                                                 mat[indices[5]]]
-    elif srank == 4 and trank == 4:
+    elif srank == 4 and trank == 4 and source is not None:
         assert not delta
         mat = {}
         for i in range(norb):
@@ -481,3 +508,4 @@ def wickfill(target: numpy.ndarray, source: numpy.ndarray, indices: List[int],
                                                             mat[indices[5]],
                                                             mat[indices[6]],
                                                             mat[indices[7]]]
+    return target
